/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * All rights reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jmatchparser.template;

import java.io.IOException;
import java.io.PrintStream;

/**
 * A formatter that calls multiple formatters in sequence.
 */
public class ChainedFormatter implements Formatter {
	private final Formatter[] formatters;
	private final String name;

	/**
	 * Create a new chained formatter.
	 * 
	 * @param name
	 *            Name of the formatter
	 * @param formatters
	 *            Formatters to call
	 */
	public ChainedFormatter(String name, Formatter... formatters) {
		this.name = name;
		this.formatters = formatters;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public String format(String content, PrintStream debugStream) throws IOException {
		for (Formatter formatter : formatters) {
			content = formatter.format(content, debugStream);
		}
		return content;
	}
}
