/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * All rights reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jmatchparser.template.engine;

public class CallStackFrame {
	private final int returnAddress;
	private final CallStackFrame nextFrame;
	private final String origName, origValue;

	public CallStackFrame(CallStackFrame nextFrame, int returnAddress) {
		this.nextFrame = nextFrame;
		this.returnAddress = returnAddress;
		origName = origValue = null;
	}

	public CallStackFrame(CallStackFrame thisFrame, String name, String oldValue) {
		if (thisFrame == null || thisFrame.containsName(name))
			throw new IllegalArgumentException();
		this.nextFrame = thisFrame;
		returnAddress = -1;
		origName = name;
		origValue = oldValue;
	}

	public boolean containsName(String name) {
		if (origName == null)
			return false;
		if (origName.equals(name))
			return true;
		// nextFrame is non-null here!
		return nextFrame.containsName(name);
	}

	public CallStackFrame restoreVariables(ParserState state) {
		CallStackFrame f = this;
		while (f.origName != null) {
			state.setLocal(f.origName, f.origValue);
			f = f.nextFrame;
		}
		return f;
	}

	public int getReturnAddress() {
		return returnAddress;
	}

	public CallStackFrame getNextFrame() {
		return nextFrame;
	}
}
