/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * All rights reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jmatchparser.template.engine;

import javax.script.Bindings;
import javax.script.ScriptEngine;
import net.sf.jmatchparser.template.CallbackFunction;

public class DefaultCallbackFunctions {
	static void addTo(ParserImpl parser) {
		parser.addCallback("replace", new CallbackFunction() {
			@Override
			public String call(String[] arguments) {
				if (arguments.length % 2 != 1)
					throw new RuntimeException("Replace needs odd number of arguments: " + arguments.length);
				String result = arguments[0];
				for (int i = 1; i < arguments.length; i += 2) {
					result = result.replaceAll(arguments[i], arguments[i + 1]);
				}
				return result;
			}
		});
		parser.addCallback("uppercase", new CallbackFunction() {
			@Override
			public String call(String[] arguments) {
				if (arguments.length != 1)
					throw new RuntimeException("Uppercase needs 1 argument: " + arguments.length);
				return arguments[0].toUpperCase();
			}
		});
		parser.addCallback("lowercase", new CallbackFunction() {
			@Override
			public String call(String[] arguments) {
				if (arguments.length != 1)
					throw new RuntimeException("Lowercase needs 1 argument: " + arguments.length);
				return arguments[0].toLowerCase();
			}
		});
		parser.addCallback("addentities", new CallbackFunction() {
			@Override
			public String call(String[] arguments) {
				if (arguments.length != 1)
					throw new RuntimeException("AddEntities needs 1 argument: " + arguments.length);
				return addEntities(arguments[0]);
			}
		});
		parser.addCallback("stripentities", new CallbackFunction() {
			@Override
			public String call(String[] arguments) {
				if (arguments.length != 1)
					throw new RuntimeException("StripEntities needs 1 argument: " + arguments.length);
				return stripEntities(arguments[0]);
			}
		});
		parser.addCallback("js", new CallbackFunction() {
			@Override
			public String call(String[] arguments) {
				if (arguments.length == 0)
					throw new RuntimeException("JS needs at least 1 argument: " + arguments.length);
				ScriptEngine engine = new javax.script.ScriptEngineManager().getEngineByName("rhino");
				if (engine == null)
					throw new RuntimeException("Rhino engine not found");
				Bindings bindings = engine.createBindings();
				for (int i = 1; i < arguments.length; i++) {
					bindings.put("param" + i, arguments[i]);
				}
				try {
					return engine.eval(arguments[0], bindings).toString();
				} catch (Exception ex) {
					throw new RuntimeException("Error while running Rhino engine", ex);
				}
			}
		});
	}

	public static String stripEntities(String string) {
		string = string.replace("&lt;", "<");
		string = string.replace("&gt;", ">");
		string = string.replace("&quot;", "\"");
		string = string.replace("&amp;", "&");
		return string;
	}

	public static String addEntities(String string) {
		string = string.replace("&", "&amp;");
		string = string.replace("<", "&lt;");
		string = string.replace(">", "&gt;");
		string = string.replace("\"", "&quot;");
		return string;
	}
}
