/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * All rights reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jmatchparser.template.engine.formatter;

import java.io.IOException;

import net.sf.jmatchparser.template.Formatter;

import org.w3c.tidy.Tidy;
import java.io.*;
import java.util.regex.Pattern;

public class JTidyFormatter implements Formatter {

	private final boolean cleanSpace;

	public JTidyFormatter(boolean cleanSpace) {
		this.cleanSpace = cleanSpace;
	}

	@Override
	public String getName() {
		return cleanSpace ? "jtidy+cleanspace" : "jtidy";
	}

	@Override
	public String format(String content, PrintStream debugStream) throws IOException {
		if (content.length() == 0)
			return content;
		Tidy tidy = new Tidy();
		tidy.setXmlOut(true);
		tidy.setWraplen(0);
		tidy.setShowWarnings(false);
		tidy.setQuiet(true);
		tidy.setShowErrors(0);
		tidy.setForceOutput(true);
		tidy.setTidyMark(false);
		tidy.setNumEntities(true);
		tidy.setRawOut(true);
		StringWriter sw = new StringWriter();
		try {
			tidy.parse(new StringReader(content), sw);
		} catch (StringIndexOutOfBoundsException ex) {
			// jTidy bug #2922337
			String bugAvoider = "<script>BugAvoider</script>";
			String bugAvoider2 = Pattern.quote("<script type=\"text/javascript\">\r\n//<![CDATA[\r\nBugAvoider\r\n//]]>\r\n</script>\r\n");
			debugStream.println(" * I caught the bug * ");
			content = content.replaceFirst("<title", bugAvoider + "<title");
			tidy.parse(new StringReader(content), sw);
			String tmp = sw.toString();
			sw = new StringWriter();
			sw.write(tmp.replaceFirst(bugAvoider2, ""));
		}
		sw.close();
		if (sw.toString().length() == 0) {
			throw new RuntimeException("Content is empty after preprocessing");
		}
		content = sw.toString();
		if (cleanSpace)
			content = content.replaceAll("[\r\n][ \t]+<", "<").replaceAll("[\r\n]+", "").replace('\t', ' ').replaceAll("  +", " ");
		content = content.replace("\r\n", "\n").replace('\r', '\n');
		if (!content.endsWith("\n"))
			content = content + "\n";
		return content;
	}
}
