/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * All rights reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jmatchparser.template.engine.formatter;

import java.io.PrintStream;

import net.sf.jmatchparser.template.Formatter;

public class TrimFormatter implements Formatter {

	private final boolean trimAll;

	public TrimFormatter(boolean trimAll) {
		this.trimAll = trimAll;
	}

	@Override
	public String getName() {
		return trimAll ? "trimall" : "trimlines";
	}

	@Override
	public String format(String content, PrintStream debugStream) {
		StringBuilder result = new StringBuilder();
		for (String line : content.split("\n")) {
			line = line.trim();
			if (trimAll)
				line = line.replace('\t', ' ').replaceAll("  +", " ");
			if (line.length() > 0 || !trimAll) {
				result.append(line).append('\n');
			}
		}
		return result.toString();
	}

}
