/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * All rights reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jmatchparser.template.engine.operation;

import net.sf.jmatchparser.template.engine.ParserState;
import net.sf.jmatchparser.template.engine.parameter.Parameter;

public class LoopForkOperation extends Operation {

	private final Label label;
	private final CounterOperation co;
	private final Parameter mincount;
	private final Parameter maxcount;

	public LoopForkOperation(String templatePosition, Label label,
			CounterOperation co, Parameter mincount, Parameter maxcount) {
		super(templatePosition);
		this.label = label;
		this.co = co;
		this.mincount = mincount;
		this.maxcount = maxcount;
	}

	@Override
	public OperationResult execute(ParserState state) {
		int min = 0, max = Integer.MAX_VALUE;
		try {
			min = Integer.parseInt(mincount.getString(state));
		} catch (NumberFormatException ex) {
		}
		if (maxcount != null) {
			try {
				max = Integer.parseInt(maxcount.getString(state));
			} catch (NumberFormatException ex) {
			}
		}
		if (min > max) {
			int tmp = max;
			max = min;
			min = tmp;
		}
		int counter = co.getCounter();
		if (counter < max) {
			// fork inside the loop
			state.getParser().enqueueState(new ParserState(state, label.getDestination()));
		}
		if (counter < min) {
			return OperationResult.DIE;
		}
		return OperationResult.CONTINUE;
	}
}
