/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * All rights reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jmatchparser.template.engine.operation;

import java.util.regex.Matcher;

import net.sf.jmatchparser.template.engine.ParserState;
import net.sf.jmatchparser.template.engine.parameter.Parameter;
import net.sf.jmatchparser.util.PatternCache;

public class MatchAnyOperation extends Operation {

	private final Parameter parameter;
	private final Label nextInstruction;

	public MatchAnyOperation(String templatePosition, Parameter parameter, Label nextInstruction) {
		super(templatePosition);
		this.parameter = parameter;
		this.nextInstruction = nextInstruction;
	}

	@Override
	public OperationResult execute(ParserState state) {
		String regex = parameter.getRegex(state);
		boolean success;
		boolean enqueued = false;
		Matcher m;

		CharSequence rest = regex.contains("\n") ? state.getRest() : state.getFirstLine();
		if (rest == null)
			rest = "";
		while (true) {
			m = PatternCache.compile("^" + regex).matcher(rest);
			success = m.find();
			if (!success)
				break;
			if (m.start() != 0)
				throw new RuntimeException();
			ParserState newState = new ParserState(state, nextInstruction.getDestination());
			parameter.applyMatch(newState, m);
			newState.discardCharacters(m.end());
			enqueued = true;
			state.getParser().enqueueState(newState);
			rest = rest.subSequence(0, m.end() - 1);
		}
		if (enqueued) {
			return OperationResult.DIE;
		} else {
			return OperationResult.FAIL;
		}
	}
}
