/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * All rights reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jmatchparser.template.engine.operation;

import java.util.ArrayList;
import java.util.List;

import net.sf.jmatchparser.template.engine.ParserState;

public class MatchXMLOperation extends Operation {

	private final boolean checkOnly;

	public MatchXMLOperation(String templatePosition, boolean checkOnly) {
		super(templatePosition);
		this.checkOnly = checkOnly;
	}

	@Override
	public OperationResult execute(ParserState state) {
		boolean success = true;
		int consumed = 0;
		CharSequence rest = state.getRest();
		if (rest == null) {
			success = false;
		} else {
			List<String> stack = new ArrayList<String>();
			for (String line : rest.toString().split("\n")) {
				boolean continuedLine = false;
				consumed += line.length() + 1;
				switch (line.charAt(0)) {
				case '<':
					if (!line.endsWith(">"))
						throw new RuntimeException("Invalid XML line: " + line);
					if (line.startsWith("</")) {
						// end tag must match start tag
						String tagName = line.substring(2, line.length() - 1);
						if (stack.size() == 0) {
							success = false;
						} else {
							String lastTag = stack.remove(stack.size() - 1);
							if (!lastTag.equals(tagName)) {
								success = false;
							}
						}
					} else {
						// add start tag
						int pos = line.indexOf(" ");
						if (pos == -1)
							pos = line.length() - 1;
						stack.add(line.substring(1, pos));
					}
					break;
				case '+':
					continuedLine = true;
					break;
				case '!': // comment
				case '[': // CDATA
				case '-': // text
				case '?': // processing instruction
					break;
				default:
					throw new RuntimeException("Invalid XML line: " + line);
				}
				if (!success || (!continuedLine && stack.size() == 0)) {
					break;
				}
			}
		}

		if (success || !checkOnly) {
			state.discardCharacters(consumed);
		}
		if (success)
			return OperationResult.CONTINUE;
		else
			return OperationResult.FAIL;
	}
}
