/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * All rights reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jmatchparser.template.engine.operation;

import net.sf.jmatchparser.template.engine.CallStackFrame;
import net.sf.jmatchparser.template.engine.ParserState;
import net.sf.jmatchparser.template.engine.parameter.Parameter;

public class SetLocalOperation extends Operation {

	private final Parameter varname;
	private final Parameter value;

	public SetLocalOperation(String templatePosition, Parameter varname, Parameter value) {
		super(templatePosition);
		this.varname = varname;
		this.value = value;
	}

	@Override
	public OperationResult execute(ParserState state) {
		String name = varname.getString(state);
		String v = value == null ? null : value.getString(state);
		CallStackFrame frame = state.getCallStack();
		if (frame == null)
			throw new RuntimeException("SETLOCAL command cannot be used outside DEFTEMPLATE");
		if (!frame.containsName(name)) {
			String oldvalue = state.getLocal(name);
			state.setCallStack(new CallStackFrame(frame, name, oldvalue));
		}
		if (v != null)
			state.setLocal(name, v);
		return OperationResult.CONTINUE;
	}
}
