/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * All rights reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jmatchparser.template.engine.operation;

import java.util.regex.Matcher;

import net.sf.jmatchparser.template.engine.ParserState;
import net.sf.jmatchparser.template.engine.parameter.Parameter;
import net.sf.jmatchparser.util.PatternCache;

public class SetOperation extends Operation {

	private final Parameter tag;
	private final Parameter value;

	public SetOperation(String templatePosition, Parameter tag, Parameter value) {
		super(templatePosition);
		this.tag = tag;
		this.value = value;
	}

	@Override
	public OperationResult execute(ParserState state) {
		String v = value.getString(state);
		Matcher m = PatternCache.compile(tag.getRegex(state)).matcher(v);
		if (!m.matches())
			return OperationResult.FAIL;
		tag.applyMatch(state, m);
		return OperationResult.CONTINUE;
	}
}
