/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * All rights reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jmatchparser.template.engine.operation;

import net.sf.jmatchparser.template.engine.ParserState;

// these operations have a side effect,
// and can only be called if there are no other
// queued states
public abstract class SideEffectOperation extends Operation {

	protected SideEffectOperation(String templatePosition) {
		super(templatePosition);
	}

	@Override
	public final OperationResult execute(ParserState state) {
		// note that this operation can be run a lot of times before
		// all states are away, because the other states may fork a lot
		// before all of them die
		if (state.getParser().getSideEffectState() == state)
			state.getParser().setSideEffectState(null);
		if (state.getParser().hasQueuedStates()) {
			if (state.getParser().getSideEffectState() != null) {
				// another operation is waiting as well; so set
				// waitingSideEffectOperation to this so that
				// the other one will fail as well
				state.getParser().setSideEffectState(state);
				return OperationResult.FAIL_BLOCKED;
			} else {
				state.getParser().setSideEffectState(state);
				// put this state to the end and wait for the others to die
				state.setNextInstruction(state.getNextInstruction() - 1);
				state.getParser().enqueueState(state);
				return OperationResult.DIE;
			}
		} else {
			if (state.getParser().getSideEffectState() != null) {
				return OperationResult.FAIL_BLOCKED;
			}
			// optimize the parser state
			state.preprocessXmlActions();
			return executeWithSideEffects(state);
		}
	}

	public abstract OperationResult executeWithSideEffects(ParserState state);
}
