/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * All rights reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jmatchparser.template.engine.parameter;

import net.sf.jmatchparser.template.engine.DefaultCallbackFunctions;
import net.sf.jmatchparser.template.engine.ParserState;

class EntityWrapperTag extends Tag {

	private final Tag baseTag;

	EntityWrapperTag(Tag baseTag) {
		this.baseTag = baseTag;
	}

	@Override
	void applyMatch(ParserState ps, String[] matches) {
		String[] m = new String[matches.length];
		for (int i = 0; i < m.length; i++) {
			m[i] = DefaultCallbackFunctions.stripEntities(matches[i]);
		}
		baseTag.applyMatch(ps, m);
	}

	@Override
	boolean isStatic() {
		return baseTag.isStatic();
	}

	@Override
	String getRegex(ParserState state) {
		if (baseTag.isParsingOnly()) {
			return baseTag.getRegex(state);
		}
		return DefaultCallbackFunctions.addEntities(baseTag.getRegex(state));
	}

	@Override
	String getString(ParserState state) {
		if (baseTag.isParsingOnly()) {
			return baseTag.getString(state);
		}
		return DefaultCallbackFunctions.addEntities(baseTag.getString(state));
	}

	@Override
	int getUsedGroupCount() {
		return baseTag.getUsedGroupCount();
	}

	@Override
	boolean isParsingOnly() {
		return baseTag.isParsingOnly();
	}
}
