/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * All rights reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jmatchparser.template.engine.parameter;

import net.sf.jmatchparser.template.engine.ParserState;

abstract class ParseTag extends Tag {

	private final Parameter regex;

	ParseTag(String regex) {
		this.regex = Parameter.parseRawParameter(regex);
	}

	@Override
	boolean isParsingOnly() {
		return true;
	}

	@Override
	int getUsedGroupCount() {
		return 1;
	}

	@Override
	boolean isStatic() {
		return regex.isStringStatic();
	}

	@Override
	String getRegex(ParserState state) {
		return "(" + regex.getString(state) + ")";
	}

	@Override
	void applyMatch(ParserState state, String[] matches) {
		if (matches.length != 1)
			throw new RuntimeException();
		applyMatch(state, matches[0]);
	}

	protected abstract void applyMatch(ParserState state, String match);

	@Override
	String getString(ParserState state) {
		throw new RuntimeException("This tag can only be used for parsing");
	}
}
