/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * All rights reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jmatchparser.template.engine.template.command.plain;

import net.sf.jmatchparser.template.engine.operation.ForkOperation;
import net.sf.jmatchparser.template.engine.operation.JumpOperation;
import net.sf.jmatchparser.template.engine.operation.Label;
import net.sf.jmatchparser.template.engine.template.PlainBlockCommandState;
import net.sf.jmatchparser.template.engine.template.MatchTemplateImpl;

public class AlternativeCommand extends ParameterlessCommand {

	private final boolean optional;

	public AlternativeCommand(boolean optional) {
		this.optional = optional;
	}

	@Override
	protected PlainBlockCommandState parse(MatchTemplateImpl template) {
		Label endLabel = new Label(), nextPartLabel = new Label(), forkLabel = new Label();
		if (optional) {
			// jump around here to make sure every path has at least one unique
			// operation
			Label dummyEndLabel = new Label();
			Label dummyContLabel = new Label();
			template.appendOperation(new ForkOperation(template.getCurrentTemplatePosition(), dummyEndLabel, false));
			template.appendOperation(new JumpOperation(template.getCurrentTemplatePosition(), dummyContLabel));
			dummyEndLabel.setDestinationToNextCommand(template);
			template.appendOperation(new JumpOperation(template.getCurrentTemplatePosition(), endLabel));
			dummyContLabel.setDestinationToNextCommand(template);
		}
		int lastForkPoint = template.getNextOperationIndex();
		template.appendOperation(new ForkOperation(template.getCurrentTemplatePosition(), forkLabel, false));
		template.appendOperation(new JumpOperation(template.getCurrentTemplatePosition(), nextPartLabel));
		forkLabel.setDestinationToNextCommand(template);
		return new AlternativeCommandState(template, lastForkPoint, nextPartLabel, endLabel);
	}

	private class AlternativeCommandState extends PlainBlockCommandState {

		private Label nextPartLabel;
		private final Label endLabel;
		private int lastForkPoint;

		public AlternativeCommandState(MatchTemplateImpl template, int lastForkPoint,
				Label nextPartLabel, Label endLabel) {
			super(template);
			this.lastForkPoint = lastForkPoint;
			this.nextPartLabel = nextPartLabel;
			this.endLabel = endLabel;
		}

		@Override
		public boolean canParse(String commandName) {
			return commandName.equals("nextalternative") || commandName.equals("nextalt") || commandName.equals("endalternative") || commandName.equals("endalt");
		}

		@Override
		public PlainBlockCommandState parseCommand(MatchTemplateImpl template,
				String commandName, String parameters) {
			if (commandName.startsWith("end")) {
				template.nopOperation(lastForkPoint);
				template.nopOperation(lastForkPoint + 1);
				endLabel.setDestinationToNextCommand(template);
				return null;
			} else {
				template.appendOperation(new JumpOperation(template.getCurrentTemplatePosition(), endLabel));
				nextPartLabel.setDestinationToNextCommand(template);
				nextPartLabel = new Label();
				Label forkLabel = new Label();
				lastForkPoint = template.getNextOperationIndex();
				template.appendOperation(new ForkOperation(template.getCurrentTemplatePosition(), forkLabel, false));
				template.appendOperation(new JumpOperation(template.getCurrentTemplatePosition(), nextPartLabel));
				forkLabel.setDestinationToNextCommand(template);
				return this;
			}
		}
	}
}
