/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * All rights reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jmatchparser.template.engine.template.command.plain;

import java.util.HashMap;
import java.util.Map;

import net.sf.jmatchparser.template.engine.template.PlainBlockCommandState;
import net.sf.jmatchparser.template.engine.template.MatchTemplateImpl;

public abstract class Command {

	private static final Map<String, Command> commands = new HashMap<String, Command>();

	static {
		// assignment commands
		commands.put("block", new XMLBlockCommand());
		commands.put("set", new SetCommand(false));
		commands.put("setexpression", new SetCommand(true));

		// control flow commands
		commands.put("loop", new LoopCommand());
		commands.put("alternative", new AlternativeCommand(false));
		commands.put("alt", new AlternativeCommand(false));
		commands.put("optionalalternative", new AlternativeCommand(true));
		commands.put("optalt", new AlternativeCommand(true));
		commands.put("optional", new OptionalCommand(false));
		commands.put("opt", new OptionalCommand(false));
		commands.put("repeated", new OptionalCommand(true));
		commands.put("checkothers", new CheckOthersCommand());
		commands.put("finishothers", new FinishOthersCommand());
		commands.put("deftemplate", new DefTemplateCommand());
		commands.put("calltemplate", new CallTemplateCommand());
		commands.put("setlocal", new SetLocalCommand());

		// CHECK commands
		commands.put("check", new MatchCommand(true, false, false));
		commands.put("checknot", new MatchCommand(true, true, false));
		commands.put("checkline", new MatchCommand(true, false, true));
		commands.put("checklinenot", new MatchCommand(true, true, true));
		commands.put("checktext", new MatchTextCommand(true, false));
		commands.put("checktextnot", new MatchTextCommand(true, true));
		commands.put("cl", new MatchCommand(true, false, true));
		commands.put("checkxml", new MatchXMLCommand(false));

		// MATCH commands
		commands.put("match", new MatchCommand(false, false, false));
		commands.put("matchline", new MatchCommand(false, false, true));
		commands.put("matchtext", new MatchTextCommand(false, false));
		commands.put("ml", new MatchCommand(false, false, true));
		commands.put("matchxml", new MatchXMLCommand(true));
		commands.put("matchany", new MatchAnyCommand());

		// WRITE comands
		commands.put("write", new WriteCommand(false));
		commands.put("writeline", new WriteCommand(true));
		commands.put("wl", new WriteCommand(true));
		commands.put("format", new FormatCommand());
		commands.put("nextpass", new NextPassCommand());

		// other commands
		commands.put("debugwrite", new DebugWriteCommand());
		commands.put("dw", new DebugWriteCommand());
		commands.put("dumpline", new DumpLineCommand());
		commands.put("dl", new DumpLineCommand());
	}

	public static Command getCommand(String commandName) {
		return commands.get(commandName);
	}

	public abstract PlainBlockCommandState parse(MatchTemplateImpl template, String parameters);
}
