/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * All rights reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jmatchparser.template.engine.template.command.plain;

import net.sf.jmatchparser.template.engine.operation.JumpOperation;
import net.sf.jmatchparser.template.engine.operation.Label;
import net.sf.jmatchparser.template.engine.operation.ReturnOperation;
import net.sf.jmatchparser.template.engine.template.MatchTemplateImpl;
import net.sf.jmatchparser.template.engine.template.PlainBlockCommandState;

public class DefTemplateCommand extends Command {

	@Override
	public PlainBlockCommandState parse(MatchTemplateImpl template, String parameters) {

		// Continue after the loop and fork inside the loop.
		// The other way round is easier to code, but will cause
		// a huge pile of queued of states to pile up if the loop repeats
		// a lot of times.

		Label startLabel = new Label(), endLabel = new Label();
		template.appendOperation(new JumpOperation(template.getCurrentTemplatePosition(), endLabel));
		startLabel.setDestinationToNextCommand(template);
		template.getDefLabels().put(parameters, startLabel);
		return new TemplateCommandState(template, endLabel);
	}

	private class TemplateCommandState extends PlainBlockCommandState {

		private final Label endLabel;

		public TemplateCommandState(MatchTemplateImpl template, Label endLabel) {
			super(template);
			this.endLabel = endLabel;
		}

		@Override
		public boolean canParse(String commandName) {
			return commandName.equals("endtemplate");
		}

		@Override
		public PlainBlockCommandState parseCommand(MatchTemplateImpl template,
				String commandName, String parameters) {
			template.appendOperation(new ReturnOperation(template.getCurrentTemplatePosition()));
			endLabel.setDestinationToNextCommand(template);
			return null;
		}
	}
}
