/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * All rights reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jmatchparser.template.engine.template.command.plain;

import net.sf.jmatchparser.template.engine.operation.CounterOperation;
import net.sf.jmatchparser.template.engine.operation.IncrementCounterOperation;
import net.sf.jmatchparser.template.engine.operation.JumpOperation;
import net.sf.jmatchparser.template.engine.operation.Label;
import net.sf.jmatchparser.template.engine.operation.LoopForkOperation;
import net.sf.jmatchparser.template.engine.parameter.Parameter;
import net.sf.jmatchparser.template.engine.template.PlainBlockCommandState;
import net.sf.jmatchparser.template.engine.template.MatchTemplateImpl;

public class LoopCommand extends Command {

	@Override
	public PlainBlockCommandState parse(MatchTemplateImpl template, String parameters) {
		// Continue after the loop and fork inside the loop.
		// The other way round is easier to code, but will cause
		// a huge pile of queued of states to pile up if the loop repeats
		// a lot of times.
		Parameter[] params = Parameter.parseParameters(parameters, 0, 2, 1, template.getSpecialTags());
		if (params[0] == null)
			params[0] = Parameter.getLiteralParameter("0");
		Label startLabel = new Label(), endLabel = new Label(), forkLabel = new Label();
		CounterOperation co = new CounterOperation(template.getCurrentTemplatePosition());
		template.appendOperation(co);
		startLabel.setDestinationToNextCommand(template);
		template.appendOperation(new LoopForkOperation(template.getCurrentTemplatePosition(), forkLabel, co, params[0], params[1]));
		template.appendOperation(new JumpOperation(template.getCurrentTemplatePosition(), endLabel));
		forkLabel.setDestinationToNextCommand(template);
		return new LoopCommandState(template, co, startLabel, endLabel);
	}

	private class LoopCommandState extends PlainBlockCommandState {

		private final CounterOperation co;
		private final Label startLabel;
		private final Label endLabel;

		public LoopCommandState(MatchTemplateImpl template, CounterOperation co, Label startLabel, Label endLabel) {
			super(template);
			this.co = co;
			this.startLabel = startLabel;
			this.endLabel = endLabel;
		}

		@Override
		public boolean canParse(String commandName) {
			return commandName.equals("endloop");
		}

		@Override
		public PlainBlockCommandState parseCommand(MatchTemplateImpl template,
				String commandName, String parameters) {
			template.appendOperation(new IncrementCounterOperation(template.getCurrentTemplatePosition(), co));
			template.appendOperation(new JumpOperation(template.getCurrentTemplatePosition(), startLabel));
			endLabel.setDestinationToNextCommand(template);
			return null;
		}
	}
}
