/*
 * Copyright (c) 2006 - 2011 Michael Schierl
 * All rights reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.jmatchparser.template.engine.template.command.plain;

import net.sf.jmatchparser.template.engine.operation.ForkOperation;
import net.sf.jmatchparser.template.engine.operation.JumpOperation;
import net.sf.jmatchparser.template.engine.operation.Label;
import net.sf.jmatchparser.template.engine.template.PlainBlockCommandState;
import net.sf.jmatchparser.template.engine.template.MatchTemplateImpl;

public class OptionalCommand extends Command {

	private final boolean repeated;

	public OptionalCommand(boolean repeated) {
		this.repeated = repeated;
	}

	@Override
	public PlainBlockCommandState parse(MatchTemplateImpl template, String parameters) {
		Label startLabel = new Label(), endLabel = new Label(), forkLabel = new Label();
		startLabel.setDestinationToNextCommand(template);
		template.appendOperation(new ForkOperation(template.getCurrentTemplatePosition(), forkLabel, false));
		template.appendOperation(new JumpOperation(template.getCurrentTemplatePosition(), endLabel));
		forkLabel.setDestinationToNextCommand(template);
		int pos = parameters.indexOf(" ");
		int pos2 = parameters.indexOf("\t");
		if (pos2 != -1 && (pos == -1 || pos > pos2))
			pos = pos2;
		String commandName, commandParameters;
		if (pos == -1) {
			commandName = parameters.toLowerCase();
			commandParameters = "";
		} else {
			commandName = parameters.substring(0, pos).toLowerCase();
			commandParameters = parameters.substring(pos + 1);
		}
		Command cmd = Command.getCommand(commandName);
		if (cmd != null) {
			PlainBlockCommandState bcs = cmd.parse(template, commandParameters);
			if (bcs != null)
				throw new RuntimeException("Optional/Repeated command cannot be used for block commands!");
		} else {
			throw new RuntimeException("Unsupported command: " + commandName);
		}
		if (repeated) {
			template.appendOperation(new JumpOperation(template.getCurrentTemplatePosition(), startLabel));
		}
		endLabel.setDestinationToNextCommand(template);
		return null;
	}

}
